@@ -1,11 +1,10 @@
+"""Module for the Accessory classes."""
 import asyncio
 import itertools
 import logging
 import struct
 import threading
 
-import ed25519
-
 from pyhap import util, SUPPORT_QR_CODE
 from pyhap.const import (
     STANDALONE_AID, HAP_REPR_AID, HAP_REPR_IID, HAP_REPR_SERVICES,
@@ -31,13 +30,7 @@ class Accessory:
 
     category = CATEGORY_OTHER
 
-    @classmethod
-    def create(cls, display_name, pincode, aid=STANDALONE_AID):
-        mac = util.generate_mac()
-        return cls(display_name, aid=aid, mac=mac, pincode=pincode)
-
-    def __init__(self, display_name, aid=None, mac=None, pincode=None,
-                 iid_manager=None, setup_id=None):
+    def __init__(self, display_name, aid=None, mac=None, pincode=None):
         """Initialise with the given properties.
 
         :param display_name: Name to be displayed in the Home app.
@@ -49,41 +42,32 @@ class Accessory:
             will assign the standalone AID to this `Accessory`.
         :type aid: int
 
-        :param mac: The MAC address of this `Accessory`, needed by HAP clients.
-            Defaults to None, in which case the `AccessoryDriver`
-            will assign a random MAC address to this `Accessory`.
-        :type mac: str
+        :param mac: Deprecated.
 
-        :param pincode: The pincode that HAP clients must prove they know in order
-            to pair with this `Accessory`. Defaults to None, in which case a random
-            pincode is generated. The pincode has the format "xxx-xx-xxx", where x is
-            a digit.
-        :type pincode: bytearray
+        :param pincode: Deprecated.
 
         :param setup_id: Setup ID can be provided, although, per spec, should be random
             every time the instance is started. If not provided on init, will be random.
             4 digit string 0-9 A-Z
         :type setup_id: str
         """
+        if mac or pincode:
+            logger.warning(
+                "The 'mac' and 'pincode' parameter are now deprecated."
+                "Assign the 'pincode' to the driver instead.")
         self.display_name = display_name
         self.aid = aid
         self.mac = mac
-        self.config_version = 2
         self.reachable = True
         self._pincode = pincode
-        self._setup_id = setup_id
         self.driver = None
         # threading.Event that gets set when the Accessory should stop.
         self.run_sentinel = None
-        self.event_loop = None
+        self.loop = None
         self.aio_stop_event = None
 
-        sk, vk = ed25519.create_keypair()
-        self.private_key = sk
-        self.public_key = vk
-        self.paired_clients = {}
         self.services = []
-        self.iid_manager = iid_manager or IIDManager()
+        self.iid_manager = IIDManager()
 
         self.add_info_service()
 
@@ -101,20 +85,8 @@ class Accessory:
         state['run_sentinel'] = None
         return state
 
-    @property
-    def setup_id(self):
-        if self._setup_id is None:
-            self._setup_id = util.generate_setup_id()
-        return self._setup_id
-
-    @property
-    def pincode(self):
-        if self._pincode is None:
-            self._pincode = util.generate_pincode()
-        return self._pincode
-
     def _set_services(self):
-        """Sets the services for this accessory.
+        """Set the services for this accessory.
 
         .. deprecated:: 2.0
            Initialize the service inside the accessory `init` method instead.
@@ -163,7 +135,7 @@ class Accessory:
         self.add_service(service)
         return service
 
-    def set_sentinel(self, run_sentinel, aio_stop_event, event_loop):
+    def set_sentinel(self, run_sentinel, aio_stop_event, loop):
         """Assign a run sentinel that can signal stopping.
 
         The run sentinel is a threading.Event object that can be used to manage
@@ -177,7 +149,7 @@ class Accessory:
         """
         self.run_sentinel = run_sentinel
         self.aio_stop_event = aio_stop_event
-        self.event_loop = event_loop
+        self.loop = loop
 
     def config_changed(self):
         """Notify the accessory about configuration changes.
@@ -192,9 +164,12 @@ class Accessory:
            (i.e. an Accessory that is contained in a Bridge),
            you should call the `config_changed` method on the Bridge.
 
+        Deprecated. Use `driver.state_change()` instead.
         """
-        self.config_version += 1
-        self.driver.config_changed()
+        logger.warning(
+            'This method is now deprecated. Use \' '
+            'driver.state_version\' instead.')
+        self.driver.state_changed()
 
     def add_service(self, *servs):
         """Add the given services to this Accessory.
@@ -232,29 +207,10 @@ class Accessory:
 
     def set_driver(self, driver):
         self.driver = driver
-
-    def add_paired_client(self, client_uuid, client_public):
-        """Adds the given client to the set of paired clients.
-
-        :param client_uuid: The client's UUID.
-        :type client_uuid: uuid.UUID
-
-        :param client_public: The client's public key (not the session public key).
-        :type client_public: bytes
-        """
-        self.paired_clients[client_uuid] = client_public
-
-    def remove_paired_client(self, client_uuid):
-        """Deletes the given client from the set of paired clients.
-
-        :param client_uuid: The client's UUID.
-        :type client_uuid: uuid.UUID
-        """
-        self.paired_clients.pop(client_uuid)
-
-    @property
-    def paired(self):
-        return len(self.paired_clients) > 0
+        if self.mac:
+            self.driver.state.mac = self.mac
+        if self._pincode:
+            self.driver.state.pincode = self._pincode
 
     def xhm_uri(self):
         """Generates the X-HM:// uri (Setup Code URI)
@@ -263,7 +219,7 @@ class Accessory:
         """
         buffer = bytearray(b'\x00\x00\x00\x00\x00\x00\x00\x00')
 
-        value_low = int(self.pincode.replace(b'-', b''), 10)
+        value_low = int(self.driver.state.pincode.replace(b'-', b''), 10)
         value_low |= 1 << 28
         struct.pack_into('>L', buffer, 4, value_low)
 
@@ -273,11 +229,12 @@ class Accessory:
         value_high = self.category >> 1
         struct.pack_into('>L', buffer, 0, value_high)
 
-        encoded_payload = base36.dumps(struct.unpack_from('>L', buffer, 4)[0]
-                                       + (struct.unpack_from('>L', buffer, 0)[0] * (1 << 32))).upper()
+        encoded_payload = base36.dumps(
+            struct.unpack_from('>L', buffer, 4)[0] +
+            (struct.unpack_from('>L', buffer, 0)[0] * (1 << 32))).upper()
         encoded_payload = encoded_payload.rjust(9, '0')
 
-        return 'X-HM://' + encoded_payload + self.setup_id
+        return 'X-HM://' + encoded_payload + self.driver.state.setup_id
 
     def get_characteristic(self, aid, iid):
         """Get the characteristic for the given IID.
@@ -317,6 +274,7 @@ class Accessory:
         For QRCode `base36`, `pyqrcode` are required.
         Installation through `pip install HAP-python[QRCode]`
         """
+        pincode = self.driver.state.pincode.decode()
         if SUPPORT_QR_CODE:
             xhm_uri = self.xhm_uri()
             print('Setup payload: {}'.format(xhm_uri), flush=True)
@@ -324,16 +282,16 @@ class Accessory:
                   flush=True)
             print(QRCode(xhm_uri).terminal(quiet_zone=2), flush=True)
             print('Or enter this code in your HomeKit app on your iOS device: '
-                  '{}'.format(self.pincode.decode()))
+                  '{}'.format(pincode))
         else:
             print('To use the QR Code feature, use \'pip install '
                   'HAP-python[QRCode]\'')
             print('Enter this code in your HomeKit app on your iOS device: {}'
-                  .format(self.pincode.decode()))
+                  .format(pincode))
 
+    @staticmethod
     def run_at_interval(seconds):
-        """Decorator that runs decorated method in a while loop, which repeats every
-        ``seconds`` until the ``Accessory.run_sentinel`` is set.
+        """Decorator that runs decorated method every x seconds, until stopped.
 
         .. code-block:: python
 
@@ -361,8 +319,7 @@ class Accessory:
         pass
 
     def stop(self):
-        """Called when the Accessory should stop what is doing and clean up any resources.
-        """
+        """Called when the Accessory should stop what is doing and clean up any resources."""
         pass
 
     # Driver
@@ -393,9 +350,9 @@ class Accessory:
 
 class AsyncAccessory(Accessory):
 
+    @staticmethod
     def run_at_interval(seconds):
-        """Decorator that runs decorated method in a while loop, which repeats every
-        ``seconds`` until the ``Accessory.aio_stop_event`` is set.
+        """Decorator that runs decorated method every x seconds, until stopped.
 
         .. code-block:: python
 
@@ -412,14 +369,13 @@ class AsyncAccessory(Accessory):
             async def _wrapper(self, *args, **kwargs):
                 while not await util.event_wait(self.aio_stop_event,
                                                 seconds,
-                                                self.event_loop):
+                                                self.loop):
                     await func(self, *args, **kwargs)
             return _wrapper
         return _repeat
 
     async def run(self):
-        """Override in the implementation if needed.
-        """
+        """Override in the implementation if needed."""
         pass
 
 
@@ -431,21 +387,21 @@ class Bridge(AsyncAccessory):
 
     category = CATEGORY_BRIDGE
 
-    def __init__(self, display_name, mac=None, pincode=None,
-                 iid_manager=None, setup_id=None):
-        aid = STANDALONE_AID
-        # A Bridge cannot be Bridge, hence talks directly to HAP clients.
-        # Thus, we need a mac.
-        mac = mac or util.generate_mac()
-        super().__init__(display_name, aid=aid, mac=mac, pincode=pincode,
-                         iid_manager=iid_manager, setup_id=setup_id)
+    def __init__(self, display_name, mac=None, pincode=None):
+        """
+        :param mac: Deprecated.
+
+        :param pincode: Deprecated.
+        """
+        super().__init__(display_name, aid=STANDALONE_AID, mac=mac,
+                         pincode=pincode)
         self.accessories = {}  # aid: acc
 
-    def set_sentinel(self, run_sentinel, aio_stop_event, event_loop):
+    def set_sentinel(self, run_sentinel, aio_stop_event, loop):
         """Set the same sentinel to all contained accessories."""
-        super().set_sentinel(run_sentinel, aio_stop_event, event_loop)
+        super().set_sentinel(run_sentinel, aio_stop_event, loop)
         for acc in self.accessories.values():
-            acc.set_sentinel(run_sentinel, aio_stop_event, event_loop)
+            acc.set_sentinel(run_sentinel, aio_stop_event, loop)
 
     def add_accessory(self, acc):
         """Add the given ``Accessory`` to this ``Bridge``.
@@ -487,16 +443,10 @@ class Bridge(AsyncAccessory):
 
         .. seealso:: Accessory.to_HAP
         """
-        hap_rep = [super().to_HAP()]
-
-        for acc in self.accessories.values():
-            hap_rep.append(acc.to_HAP())
-
-        return hap_rep
+        return [acc.to_HAP() for acc in (super(), *self.accessories.values())]
 
     def get_characteristic(self, aid, iid):
-        """.. seealso:: Accessory.to_HAP
-        """
+        """.. seealso:: Accessory.to_HAP"""
         if self.aid == aid:
             return self.iid_manager.get_obj(iid)
 
@@ -507,23 +457,21 @@ class Bridge(AsyncAccessory):
         return acc.get_characteristic(aid, iid)
 
     async def _wrap_in_thread(self, method):
-        """Coroutine which starts the given method in a thread.
-        """
-        # Not going through event_loop.run_in_executor, because this thread may never
+        """Coroutine which starts the given method in a thread."""
+        # Not going through loop.run_in_executor, because this thread may never
         # terminate.
         threading.Thread(target=method).start()
 
     async def run(self):
-        """Schedule tasks for each of the accessories' run method.
-        """
+        """Schedule tasks for each of the accessories' run method."""
         tasks = []
         for acc in self.accessories.values():
             if isinstance(acc, AsyncAccessory):
-                task = self.event_loop.create_task(acc.run())
+                task = self.loop.create_task(acc.run())
             else:
-                task = self.event_loop.create_task(self._wrap_in_thread(acc.run))
+                task = self.loop.create_task(self._wrap_in_thread(acc.run))
             tasks.append(task)
-        await asyncio.gather(*tasks, loop=self.event_loop)
+        await asyncio.gather(*tasks, loop=self.loop)
 
     def stop(self):
         """Calls stop() on all contained accessories."""
