@@ -68,12 +68,12 @@ class HAP_CRYPTO:
 
 
 def _pad_tls_nonce(nonce, total_len=HAP_CRYPTO.TLS_NONCE_LEN):
-    """ Pads a nonce with zeroes so that total_len is reached. """
+    """Pads a nonce with zeroes so that total_len is reached."""
     return nonce.rjust(total_len, b"\x00")
 
 
 def hap_hkdf(key, salt, info):
-    """ Just a shorthand. """
+    """Just a shorthand."""
     return HKDF(key, HAP_CRYPTO.HKDF_KEYLEN, salt, HAP_CRYPTO.HKDF_HASH, context=info)
 
 
@@ -86,11 +86,11 @@ class NotAllowedInStateException(Exception):
 
 
 class HAPServerHandler(BaseHTTPRequestHandler):
-    """ Manages HAP connection state and handles incoming HTTP requests. """
+    """Manages HAP connection state and handles incoming HTTP requests."""
 
     # Mapping from paths to methods that handle them.
     HANDLERS = {
-        "POST":  {
+        "POST": {
             "/pair-setup": "handle_pairing",
             "/pair-verify": "handle_pair_verify",
             "/pairings": "handle_pairings",
@@ -132,7 +132,7 @@ class HAPServerHandler(BaseHTTPRequestHandler):
         @type accessory_handler: AccessoryDriver
         """
         self.accessory_handler = accessory_handler
-        self.accessory = self.accessory_handler.accessory
+        self.state = self.accessory_handler.state
         self.enc_context = None
         self.is_encrypted = False
         # Redirect separate handlers to the dispatch method
@@ -153,7 +153,7 @@ class HAPServerHandler(BaseHTTPRequestHandler):
         @param client_public: The client's session public key.
         @type client_public: bytes
 
-        @param private_key: The accessory's session private key.
+        @param private_key: The state's session private key.
         @type private_key: bytes
 
         @param shared_key: The resulted session key.
@@ -211,7 +211,7 @@ class HAPServerHandler(BaseHTTPRequestHandler):
 
     def handle_pairing(self):
         """Handles arbitrary step of the pairing process."""
-        if self.accessory.paired:
+        if self.state.paired:
             raise NotAllowedInStateException
 
         length = int(self.headers["Content-Length"])
@@ -342,11 +342,11 @@ class HAPServerHandler(BaseHTTPRequestHandler):
         output_key = hap_hkdf(long_to_bytes(session_key),
                               self.PAIRING_5_SALT, self.PAIRING_5_INFO)
 
-        server_public = self.accessory.public_key.to_bytes()
-        mac = self.accessory.mac.encode()
+        server_public = self.state.public_key.to_bytes()
+        mac = self.state.mac.encode()
 
         material = output_key + mac + server_public
-        private_key = self.accessory.private_key
+        private_key = self.state.private_key
         server_proof = private_key.sign(material)
 
         message = tlv.encode(HAP_TLV_TAGS.USERNAME, mac,
@@ -375,7 +375,7 @@ class HAPServerHandler(BaseHTTPRequestHandler):
 
         Pair verify is session negotiation.
         """
-        if not self.accessory.paired:
+        if not self.state.paired:
             raise NotAllowedInStateException
 
         length = int(self.headers["Content-Length"])
@@ -404,9 +404,9 @@ class HAPServerHandler(BaseHTTPRequestHandler):
             # Key is hashed before being returned, we don't want it; This fixes that.
             lambda x: x)
 
-        mac = self.accessory.mac.encode()
+        mac = self.state.mac.encode()
         material = public_key.serialize() + mac + client_public
-        server_proof = self.accessory.private_key.sign(material)
+        server_proof = self.state.private_key.sign(material)
 
         output_key = hap_hkdf(shared_key, self.PVERIFY_1_SALT, self.PVERIFY_1_INFO)
 
@@ -445,7 +445,7 @@ class HAPServerHandler(BaseHTTPRequestHandler):
             + self.enc_context["public_key"].serialize()
 
         client_uuid = uuid.UUID(str(client_username, "ascii"))
-        perm_client_public = self.accessory.paired_clients.get(client_uuid)
+        perm_client_public = self.state.paired_clients.get(client_uuid)
         if perm_client_public is None:
             logger.debug("Client %s attempted pair verify without being paired first.",
                          client_uuid)
@@ -493,8 +493,7 @@ class HAPServerHandler(BaseHTTPRequestHandler):
             raise UnprivilegedRequestException
 
         # Check that char exists and ...
-        params = parse_qs(
-                    urlparse(self.path).query)
+        params = parse_qs(urlparse(self.path).query)
         chars = self.accessory_handler.get_characteristics(params["id"][0].split(","))
 
         data = json.dumps(chars).encode("utf-8")
@@ -507,11 +506,11 @@ class HAPServerHandler(BaseHTTPRequestHandler):
         if not self.is_encrypted:
             raise UnprivilegedRequestException
 
-        # TODO: assert self.headers["authorization"] == accessory.pincode
+        # TODO: assert self.headers["authorization"] == state.pincode
         data_len = int(self.headers["Content-Length"])
         assert data_len > 0
         requested_chars = json.loads(
-                              self.rfile.read(data_len).decode("utf-8"))
+            self.rfile.read(data_len).decode("utf-8"))
 
         chars = self.accessory_handler.set_characteristics(requested_chars,
                                                            self.client_address)
@@ -543,7 +542,7 @@ class HAPServerHandler(BaseHTTPRequestHandler):
         client_public = tlv_objects[HAP_TLV_TAGS.PUBLIC_KEY]
         client_uuid = uuid.UUID(str(client_username, "utf-8"))
         should_confirm = self.accessory_handler.pair(
-                             client_uuid, client_public)
+            client_uuid, client_public)
         if not should_confirm:
             self.send_response(500)
             return
@@ -680,14 +679,12 @@ class HAPSocket(socket.socket):
                 self.num_in_recv += actual_len
                 if self.num_in_recv == self.curr_in_total:
                     # We read a whole block. Decrypt it and append it to the result.
-                    nonce = _pad_tls_nonce(
-                                 struct.pack("Q", self.in_count))
+                    nonce = _pad_tls_nonce(struct.pack("Q", self.in_count))
                     # Note we are removing the mac length from the total length
                     block_length = self.curr_in_total - self.in_cipher.tagLength
                     plaintext = self.in_cipher.open(
-                                    nonce,
-                                    bytearray(self.curr_in_block),
-                                    struct.pack("H", block_length))
+                        nonce, bytearray(self.curr_in_block),
+                        struct.pack("H", block_length))
                     result += plaintext
                     self.in_count += 1
                     self.curr_in_block = None
@@ -713,10 +710,8 @@ class HAPSocket(socket.socket):
         while offset < total:
             length = min(total - offset, self.MAX_BLOCK_LENGTH)
             length_bytes = struct.pack("H", length)
-            block = bytearray(
-                         data[offset: offset + length])
-            nonce = _pad_tls_nonce(
-                       struct.pack("Q", self.out_count))
+            block = bytearray(data[offset: offset + length])
+            nonce = _pad_tls_nonce(struct.pack("Q", self.out_count))
             ciphertext = length_bytes \
                 + self.out_cipher.seal(nonce, block, length_bytes)
             offset += length
